#!/usr/bin/python3

import os
import unittest

from test.test_base import TestBase


class TestUntrusted(TestBase):

    def setUp(self):
        TestBase.setUp(self)
        self.rootdir = self.make_fake_aptroot(
            template=os.path.join(self.testdir, "root.untrusted"))
        self.log = os.path.join(
            self.rootdir, "var", "log", "unattended-upgrades",
            "unattended-upgrades.log")


if __name__ == "__main__":
    # do not setup logging in here or the test will break
    unittest.main()
